﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Reflection;
using System.Diagnostics;
using System.Windows.Media;
using FLY.OBJComponents.IService;

namespace FLY.Weight.Common
{
    public class WeighterC:INotifyPropertyChanged,IPropertyOpt
    {
        public WeighterC(string nam, int binCnt)
        {
            Name = nam;
            BinCnt = binCnt;
            BinsBinding();
        }

        public event PropertyChangedEventHandler PropertyChanged;

        #region IWeighterC实现

        public string Name
        {
            get;
            set;
        }
        /// <summary>
        /// 名称:A,B,C,D,E
        /// </summary>
        public string Number { get { return Name; } }
        /// <summary>
        /// 颜色
        /// </summary>
        public Brush Color { get; set; }
        /// <summary>
        /// 料仓数
        /// </summary>
        public int BinCnt { get; protected set; }

        #endregion


        #region 辅助代码生成
        #endregion



        /// <summary>
        /// 料仓，混料参数 绑定到 对应的队列
        /// </summary>
        void BinsBinding()
        {
            Ingredients.Clear();

            for (int i = 0; i < BinCnt; i++)
            {
                int no = i + 1;
                IngredientCell ingredient = new IngredientCell();
                ingredient.Number = no.ToString();
                Misc.BindingOperations.SetBinding(this, "MixPSet_" + no.ToString(), ingredient, "MixPSet", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MixPDisp_" + no.ToString(), ingredient, "MixPDisp", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MixCumPercent_" + no.ToString(), ingredient, "MixCumPercent", Misc.BindingOperations.BindingMode.TwoWay);

                Misc.BindingOperations.SetBinding(this, "MixSet_" + no.ToString(), ingredient, "MixSet", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MixDisp_" + no.ToString(), ingredient, "MixDisp", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MixCum_" + no.ToString(), ingredient, "MixCum", Misc.BindingOperations.BindingMode.TwoWay);

                Ingredients.Add(ingredient);
            }

            IngredientParams.Clear();

            for (int i = 0; i < BinCnt; i++)
            {
                int no = i + 1;
                IngredientParamCell ingredientParam = new IngredientParamCell();
                ingredientParam.Number = no.ToString();
                Misc.BindingOperations.SetBinding(this, "FeedingDeviation_" + no.ToString(), ingredientParam, "FeedingDeviation", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "FeedSet_" + no.ToString(), ingredientParam, "FeedSet", Misc.BindingOperations.BindingMode.TwoWay);

                Misc.BindingOperations.SetBinding(this, "InitTime_" + no.ToString(), ingredientParam, "InitTime", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MinTime_" + no.ToString(), ingredientParam, "MinTime", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "StableTime_" + no.ToString(), ingredientParam, "StableTime", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "BinAlarmSet_" + no.ToString(), ingredientParam, "BinAlarmSet", Misc.BindingOperations.BindingMode.TwoWay);

                IngredientParams.Add(ingredientParam);
            }
        }

        public string[] GetSyncPropNames()
        {
            return null;
        }

        public string[] GetNoSyncPropNames()
        {
            return new string[] {
                "Ingredients",
                "IngredientParams"
            };
        }
        #region 混料料仓队列

        public List<IngredientCell> Ingredients { get; } = new List<IngredientCell>();


        #endregion
        #region 混料参数队列

        public List<IngredientParamCell> IngredientParams { get; } = new List<IngredientParamCell>();
        #endregion
    }

    public class IngredientCell : INotifyPropertyChanged
    {
        /// <summary>
        /// 颜色
        /// </summary>
        public Brush Color { get; set; }


        /// <summary>
        /// 序号
        /// </summary>
        public string Number { get; set; }


        private bool mixLight;
        /// <summary>
        /// 混料 进行中
        /// </summary>
        public bool MixLight
        {
            get { return mixLight; }
            set
            {
                if (mixLight != value)
                {
                    mixLight = value;
                    NotifyPropertyChanged("MixLight");
                }
            }
        }
        private float mixPSet;
        /// <summary>
        /// 比例设定
        /// </summary>
        public float MixPSet
        {
            get { return mixPSet; }
            set
            {
                if (mixPSet != value)
                {
                    mixPSet = value;
                    NotifyPropertyChanged("MixPSet");
                }
            }
        }

        private float mixPDisp;
        /// <summary>
        /// 比例显示
        /// </summary>
        public float MixPDisp
        {
            get { return mixPDisp; }
            set
            {
                if (mixPDisp != value)
                {
                    mixPDisp = value;
                    NotifyPropertyChanged("MixPDisp");
                }
            }
        }


        private float mixCumPercent;
        /// <summary>
        /// 累计重量比例
        /// </summary>
        public float MixCumPercent
        {
            get { return mixCumPercent; }
            set
            {
                if (mixCumPercent != value)
                {
                    mixCumPercent = value;
                    NotifyPropertyChanged("MixCumPercent");
                }
            }
        }
        private float mixset;
        /// <summary>
        /// 重量设定
        /// </summary>
        public float MixSet
        {
            get { return mixset; }
            set
            {
                if (mixset != value)
                {
                    mixset = value;
                    NotifyPropertyChanged("MixSet");
                }
            }
        }

        private float mixdisp;
        /// <summary>
        /// 重量显示
        /// </summary>
        public float MixDisp
        {
            get { return mixdisp; }
            set
            {
                if (mixdisp != value)
                {
                    mixdisp = value;
                    NotifyPropertyChanged("MixDisp");
                }
            }
        }

        private float mixcum;
        /// <summary>
        /// 累计重量
        /// </summary>
        public float MixCum
        {
            get { return mixcum; }
            set
            {
                if (mixcum != value)
                {
                    mixcum = value;
                    NotifyPropertyChanged("MixCum");
                }
            }
        }


        public IngredientCell()
        {

        }

        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class IngredientParamCell : INotifyPropertyChanged
    {
        /// <summary>
        /// 序号
        /// </summary>
        public string Number { get; set; }


        private float feedingDeviation;
        /// <summary>
        /// 补料偏差
        /// </summary>
        public float FeedingDeviation
        {
            get { return feedingDeviation; }
            set
            {
                if (feedingDeviation != value)
                {
                    feedingDeviation = value;
                    NotifyPropertyChanged("FeedingDeviation");
                }
            }
        }

        private float feedSet;
        /// <summary>
        /// 加料设定
        /// </summary>
        public float FeedSet
        {
            get { return feedSet; }
            set
            {
                if (feedSet != value)
                {
                    feedSet = value;
                    NotifyPropertyChanged("FeedSet");
                }
            }
        }


        private float initTime;
        /// <summary>
        /// 初始时间
        /// </summary>
        public float InitTime
        {
            get { return initTime; }
            set
            {
                if (initTime != value)
                {
                    initTime = value;
                    NotifyPropertyChanged("InitTime");
                }
            }
        }
        private float minTime;
        /// <summary>
        /// 最小时间
        /// </summary>
        public float MinTime
        {
            get { return minTime; }
            set
            {
                if (minTime != value)
                {
                    minTime = value;
                    NotifyPropertyChanged("MinTime");
                }
            }
        }

        private float stableTime;
        /// <summary>
        /// 稳重时间
        /// </summary>
        public float StableTime
        {
            get { return stableTime; }
            set
            {
                if (stableTime != value)
                {
                    stableTime = value;
                    NotifyPropertyChanged("StableTime");
                }
            }
        }

        private float binAlarmSet;
        /// <summary>
        /// 料仓报警设定
        /// </summary>
        public float BinAlarmSet
        {
            get { return binAlarmSet; }
            set
            {
                if (binAlarmSet != value)
                {
                    binAlarmSet = value;
                    NotifyPropertyChanged("BinAlarmSet");
                }
            }
        }


        public IngredientParamCell()
        {

        }

        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }

}
